#region Copyright
/*
 * Copyright 2005 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Reflection;
using System.Collections;
using System.Runtime.Remoting.Proxies;
using System.Runtime.Remoting.Messaging;
using Seasar.Framework.Aop.Impl;
using Seasar.Framework.Util;
using System.Runtime.Remoting;

namespace Seasar.Framework.Aop.Proxy
{
    /// <summary>
    /// AopProxy
    /// </summary>
    /// <remarks>
    /// ߓIvNVɂAOPĂ܂B
    /// </remarks>
    [Serializable]
    public sealed class AopProxy : RealProxy
    {
        /// <summary>
        /// ߓIvNV쐬CX^X
        /// </summary>
        private object target_;

        /// <summary>
        /// KpAspect
        /// </summary>
        private IAspect[] aspects_;

        /// <summary>
        /// ߓIvNV쐬^
        /// </summary>
        private Type type_;

        /// <summary>
        /// \bhƂ̃NX̃CX^XS2ReiɊւ
        /// </summary>
        private Hashtable parameters_;

        public Type TargetType
        {
            get { return type_; }
        }
        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="type">Aspect^[Qbǧ^Type</param>
        /// <param name="aspects">KAspect</param>
        /// <param name="parameters">p[^</param>
        /// <param name="target">Aspect^[Qbg</param>
        public AopProxy(Type type,IAspect[] aspects,Hashtable parameters, object target) : base(type) 
        {
            type_       = type;
            target_     = target;
            aspects_    = aspects;
            parameters_ = parameters;
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="type">Aspect^[Qbǧ^Type</param>
        /// <param name="aspects">KAspect</param>
        /// <param name="parameters">p[^</param>
        public AopProxy(Type type,IAspect[] aspects,Hashtable parameters)
            : this(type,aspects,parameters,null)
        {
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="type">Aspect^[Qbǧ^Type</param>
        /// <param name="aspects">KAspect</param>
        public AopProxy(Type type,IAspect[] aspects)
            : this(type,aspects,null)
        {
        }

        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="type">Aspect^[Qbǧ^Type</param>
        public AopProxy(Type type)
            : this(type,null)
        {
        }

        /// <summary>
        /// ߓIvNVԂ
        /// </summary>
        /// <returns>ߓIvNṼCX^X</returns>
        public object Create()
        {
            return this.GetTransparentProxy();
        }

        /// <summary>
        /// ߓIvNVԂ
        /// </summary>
        /// <param name="argTypes">ߓIvNV̑ΏۂƂȂNX̃RXgN^̈̌^̃Xg</param>
        /// <param name="args">ߓIvNV̑ΏۂƂȂNX̃RXgN^̈̃Xg</param>
        /// <returns>ߓIvNṼCX^X</returns>
        public object Create(Type[] argTypes,object[] args)
        {
            ConstructorInfo constructor = ClassUtil.GetConstructorInfo(type_,argTypes);
            target_ = ConstructorUtil.NewInstance(constructor,args);
            return this.GetTransparentProxy();
        }

        /// <summary>
        /// ߓIvNVԂ
        /// </summary>
        /// <param name="argTypes">ߓIvNV̑ΏۂƂȂNX̃RXgN^̈̌^̃Xg</param>
        /// <param name="args">ߓIvNV̑ΏۂƂȂNX̃RXgN^̈̃Xg</param>
        /// <param name="targetType">ߓIvNV̑ΏۂƂȂNX̌^</param>
        /// <returns>ߓIvNṼCX^X</returns>
        public object Create(Type[] argTypes,object[] args,Type targetType)
        {
            ConstructorInfo constructor = ClassUtil.GetConstructorInfo(targetType,argTypes);
            target_ = ConstructorUtil.NewInstance(constructor,args);
            return this.GetTransparentProxy();
        }

        #region RealProxy o

        /// <summary>
        /// AopProxyʂIuWFNg̃\bhsƂ̃\bhĂ΂܂
        /// </summary>
        /// <param name="msg">IMessage</param>
        /// <returns>IMessage</returns>
        /// <seealso="System.Runtime.Remoting.Proxies">System.Runtime.Remoting.Proxies</seealso>
        public override IMessage Invoke(IMessage msg) 
        {
            if(target_ == null)
            {
                if(!type_.IsInterface) target_ = Activator.CreateInstance(type_);
                if(target_==null) target_ = new object();
            }

            IMethodMessage methodMessage = msg as IMethodMessage;
            MethodBase method = methodMessage.MethodBase;

            ArrayList interceptorList = new ArrayList();

            if(aspects_ != null)
            {
                // `ꂽAspectInterceptor̃Xg̍쐬
                foreach(IAspect aspect in aspects_)
                {
                    IPointcut pointcut = aspect.Pointcut;
                    // IPointcutAdvice(Interceptor)}邩mF
                    if(pointcut == null || pointcut.IsApplied(method)) 
                    {
                        // AspectKpꍇ
                        interceptorList.Add(aspect.MethodInterceptor);
                    }
                }
            }

            Object ret = null;

            object[] methodArgs = null;

            if(interceptorList.Count == 0)
            {
                methodArgs = methodMessage.Args;
                
                try
                {
                    //Interceptor}Ȃꍇ
                    ret = method.Invoke(target_, methodArgs);
                }
                catch (TargetInvocationException ex)
                {
                    // InnerExceptionStackTraceۑ
                    ExceptionUtil.SaveStackTraceToRemoteStackTraceString(ex.InnerException);

                    // InnerExceptionthrow
                    throw ex.InnerException;
                }
            }
            else
            {
                // Interceptor}ꍇ
                IMethodInterceptor[] interceptors = (IMethodInterceptor[])
                    interceptorList.ToArray(typeof(IMethodInterceptor));

                IMethodInvocation invocation = new MethodInvocationImpl(target_,
                    method,methodMessage.Args,interceptors,parameters_);

                ret = interceptors[0].Invoke(invocation);

                methodArgs = invocation.Arguments;
            }

            IMethodReturnMessage mrm = new ReturnMessage(ret, methodArgs, methodArgs.Length,
                methodMessage.LogicalCallContext, (IMethodCallMessage)msg);
            
            return mrm;
        }

        #endregion

    }   // AopProxy
}
